@@ -7,13 +7,16 @@
  * Support : support@synerty.com
 """
 import logging
+import typing
+import weakref
 from collections import deque, defaultdict
 from typing import Callable, Deque, Dict
 
 from twisted.internet.interfaces import IPushProducer
-from txws import WebSocketProtocol
 from zope.interface import implementer
 
+from vortex.DeferUtil import nonConcurrentMethod
+
 logger = logging.getLogger(name=__name__)
 
 
@@ -31,40 +34,89 @@ def _format_size(size):
 @implementer(IPushProducer)
 class VortexWritePushProducer(object):
     WARNING_DATA_LENGTH = 50 * 1024 * 1024
-    ERROR_DATA_LENGTH = 50 * 1024 * 1024
+    ERROR_DATA_LENGTH = 500 * 1024 * 1024
+    WRITE_CHUNK_SIZE = 128 * 1024
+
+    __memoryLoggingRefs = None
+    __memoryLoggingEnabled = False
+
+    @classmethod
+    def setupMemoryLogging(cls) -> None:
+        cls.__memoryLoggingRefs = []
+        cls.__memoryLoggingEnabled = True
+
+    @classmethod
+    def memoryLoggingDump(cls, top=10,
+                          msgs=1) -> typing.List[typing.Tuple[str, int, int]]:
+        if not cls.__memoryLoggingRefs:
+            return []
+
+        # Filter out expired items
+        cls.__memoryLoggingRefs = list(filter(lambda o: o(), cls.__memoryLoggingRefs))
+
+        results = []
+        for producerRef in cls.__memoryLoggingRefs:
+            producer = producerRef()
+            if not producer:
+                continue
+
+            queueCount = sum([len(q) for q in producer._queueByPriority.values()])
+
+            results.append((producer._remoteVortexName,
+                            queueCount,
+                            producer._queuedDataLen))
+
+        data = sorted(results, key=lambda x: x[2], reverse=True)
+
+        return list(filter(lambda x: x[2] >= msgs, data))[:top]
 
     def __init__(self, transport,
                  stopProducingCallback: Callable,
-                 remoteVortexName: str = 'Pending'):
+                 remoteVortexName: str = 'Pending',
+                 writeWholeFrames=False):
+        if VortexWritePushProducer.__memoryLoggingEnabled:
+            VortexWritePushProducer.__memoryLoggingRefs.append(weakref.ref(self))
+
         self._transport = transport
         self._stopProducingCallback = stopProducingCallback
         self._remoteVortexName = remoteVortexName
+        self._writeWholeFrames = writeWholeFrames
 
         self._paused = False
+        self._pausedDeferred = None
         self._writingInProgress = False
+        self._writingFrameInProgress = False
         self._closed = False
         self._queuedDataLen = 0
         self._queueByPriority: Dict[int, Deque] = defaultdict(deque)
 
-        # The websocket protocol likes one vortexmsg per frame
-        self._useFraming = isinstance(transport, WebSocketProtocol)
+        self._currentlyWritingData: typing.Optional[bytes] = None
+        self._currentlyWritingDataOffset: int = 0
 
     def setRemoteVortexName(self, remoteVortexName: str):
         self._remoteVortexName = remoteVortexName
 
+    @property
+    def _canContinue(self):
+        return not self._paused \
+               and not self._startWritingFrame.running \
+               and [q for q in self._queueByPriority.values() if q]
+
+    @nonConcurrentMethod
     def _startWriting(self):
-        # ---------------
-        # Write in progress logic.
-        # We should only have one write loop at a time
-        if self._writingInProgress:
-            return
-        self._writingInProgress = True
+        if self._currentlyWritingData is not None:
+            self._startWritingFrame()
 
-        # Send the messages in order of priority
-        for priority in sorted(self._queueByPriority):
-            queue = self._queueByPriority[priority]
+        while self._canContinue:
+            # Send the messages in order of priority
+            for priority in sorted(self._queueByPriority):
+                if not self._canContinue:
+                    return
+
+                queue = self._queueByPriority[priority]
+                if not queue:
+                    continue
 
-            while queue and not self._paused:
                 data = queue.popleft()
                 preLen = self._queuedDataLen
                 self._queuedDataLen -= len(data)
@@ -81,10 +133,39 @@ class VortexWritePushProducer(object):
                         self._remoteVortexName,
                         _format_size(self._queuedDataLen))
 
-                self._transport.write(data)
-                self._transport.write(b'.')
+                if self._writeWholeFrames:
+                    self._transport.write(data)
+                    self._transport.write(b'.')
 
-        self._writingInProgress = False
+                else:
+                    self._currentlyWritingData = data
+                    self._currentlyWritingDataOffset = 0
+                    self._startWritingFrame()
+
+    @nonConcurrentMethod
+    def _startWritingFrame(self):
+        # ---------------
+        # Write in progress logic.
+        # We should only have one write loop at a time
+        if self._writingFrameInProgress:
+            return
+        self._writingFrameInProgress = True
+
+        offset = self._currentlyWritingDataOffset
+        data = self._currentlyWritingData
+        while not self._paused and offset < len(data):
+            self._transport.write(data[offset:offset + self.WRITE_CHUNK_SIZE])
+            offset += self.WRITE_CHUNK_SIZE
+
+        if len(data) <= offset:
+            self._currentlyWritingDataOffset = 0
+            self._currentlyWritingData = None
+            self._transport.write(b'.')
+
+        else:
+            self._currentlyWritingDataOffset = offset
+
+        self._writingFrameInProgress = False
 
     def pauseProducing(self):
         """
